// EMF Detector for 7-Segment LED Display v1.0
//
// original code/project by Aaron ALAI - aaronalai1@gmail.com
// modified for use w/ LED bargraph by Collin Cunningham - collin@makezine.com
// modified again by ComputerGeek for instructable & 7-segment display.

// a header for outgoing serial messages
#define MSGHEADER 0xAB

// number of analog pins to read
#define NUMPINS 1

// read at most 100 times a second
#define READPERIOD 5

#define NUMREADINGS 15 // raise this number to increase data smoothing

// Serial stuff
byte serialMsg[3];
int whichPin;
unsigned long lastReadWrite;

// Sensing Stuff
int senseLimit = 15; // raise this number to decrease sensitivity (up to 1023 max)
int probePin = 5; // analog 5
int val = 0; // reading from probePin


// variables for smoothing

int readings[NUMREADINGS];                // the readings from the analog input
int index = 0;                            // the index of the current reading
int total = 0;                            // the running total
int average = 0;                          // final average of the probe reading

//CHANGE THIS TO affect the speed of the updates for numbers. Lower the number the faster it updates.
int updateTime = 40;

void setup() {

  // Serial init
  Serial.begin(57600);
  serialMsg[0] = serialMsg[1] = serialMsg[2] = 0x00;
  whichPin = 0;
  lastReadWrite = millis();
  
  //Sensor init
  for (int i = 0; i < NUMREADINGS; i++)
    readings[i] = 0;                      // initialize all the readings to 0
}

void loop() {
  
  if(millis()-lastReadWrite > READPERIOD){
    val = analogRead(probePin);  // take a reading from the probe
  
    if(val >= 1){                // if the reading isn't zero, proceed
      val = constrain(val, 1, senseLimit);  // turn any reading higher than the senseLimit value into the senseLimit value
      val = map(val, 1, senseLimit, 1, 1023);  // remap the constrained value within a 1 to 1023 range
  
      total -= readings[index];               // subtract the last reading
      readings[index] = val; // read from the sensor
      total += readings[index];               // add the reading to the total
      index = (index + 1);                    // advance to the next index
  
      if (index >= NUMREADINGS)               // if we're at the end of the array...
        index = 0;                            // ...wrap around to the beginning
      
      // Calculate and clean Signal values
      average = total / NUMREADINGS;          // calculate the average
  
      // send data
      // send 3 bytes per value: HHHH_HHHH AAAA_VVVV VVVV_VVVV
      // where HHHH_HHHH = 8-bit fixed header
      //       AAAA = which pin [0,15]
      //       VVVV VVVV_VVVV = 12 bits of data [0,4095]
      short readValue = average;
      serialMsg[0] = (MSGHEADER&0xFF);
      serialMsg[1] = ((whichPin<<4)&0xF0) | ((readValue>>8)&0x0F);
      serialMsg[2] = (readValue&0xFF);
  
      Serial.write(serialMsg[0]);
      Serial.write(serialMsg[1]);
      Serial.write(serialMsg[2]);
  
      Serial.flush();
      whichPin = (whichPin+1)%NUMPINS;
      // use millis() because flush could take longer
      lastReadWrite = millis();
    }
  }

}



